-module(commstime_print).
-export([main/1, ring_element/2]).

-define(ELEMENTS, 4).

next_token(0) -> 0;
next_token(Token) -> Token + 1.

ring_element(0, _) -> 
   done;
ring_element(_, Next) ->
   receive
      Token ->
         Next ! next_token(Token),
         ring_element(Token, Next)
   end.

ring_inner(0, Token, Next) ->
   io:fwrite("end~n"),
   io:fwrite("token ~b~n", [Token]),
   {_, Time1} = statistics(runtime),
   {_, Time2} = statistics(wall_clock),
   U1 = Time1 * 1000/Token,
   U2 = Time2 * 1000/Token,
   io:format("commstime =~p (~p) microseconds~n",
      [U1, U2]),
   Next ! 0,
   receive
    _ -> done
   end;
ring_inner(Cycles, Token, Next) ->
     Next ! Token + 1,
   receive
      NewToken ->
        ring_inner(Cycles - 1, NewToken, Next)
   end.

commstime_print(1, Cycles, Next) ->
   Next ! 1,
   receive
      Token ->
        io:fwrite("start~n"),
        statistics(runtime),
        statistics(wall_clock),
        ring_inner(Cycles, Token, Next)
   end;
commstime_print(N, Cycles, Last) ->
   Next = spawn(commstime_print, ring_element, [1, Last]),
   commstime_print(N - 1, Cycles, Next).

ring_root(Cycles) ->
   commstime_print(?ELEMENTS, Cycles, self()).

main([Arg]) ->
   Cycles = list_to_integer(Arg),
   ring_root(Cycles),
   erlang:halt().

