module ChanState where 

import List as L
import Control.Monad.State
import Data.Map as M

import Parser.ErrM

import Backend.Abs

import GenErr
import GeneratorState
import Help
import Includes

addChan :: BEExp -> GenErr ()
addChan chanDecl = 
    getChanState >>= \st -> modifyChanState st { chsChans =  
         M.insert chanDecl ((BEIdent ""), (BEIdent "")) (chsChans st) }

addChanArray :: BEExp -> BEExp -> GenErr ()
addChanArray var x = case x of 
  BEEConst a -> case a of
     BEConstLiteralInt n -> loopAddChanArray var n
     _ -> lift $ Bad $ "Channel array size has to be set by an int" ++ (show var) ++ (show x)
  _ -> lift $ Bad $ "Channel array size has to be set by an int" ++ (show var) ++ (show x)

getChanList :: GenErr [BEExp]
getChanList = getChanState >>= \st -> return $ M.keys $ chsChans st

loopAddChanArray :: BEExp -> Integer -> GenErr ()
loopAddChanArray var 0 = addChan $ BEEArray var $ BEEConst $ BEConstLiteralInt 0
loopAddChanArray var n = do
    addChan $ BEEArray var $ BEEConst $ BEConstLiteralInt n
    loopAddChanArray var (n-1)

addSendToChanState :: BEExp -> GenErr ()
addSendToChanState chan = do 
    send <- getCurrentProc
    ch <- getChanState
    let a = M.lookup chan (chsChans ch)
    case a of
        Just d -> modifySendInChanState chan send d ch
        Nothing -> lift $ Bad $ "Channel not decleared" ++ show chan

modifySendInChanState :: BEExp -> BEIdent -> (BEIdent, BEIdent) -> ChanState -> GenErr ()
modifySendInChanState chan send (r, s) ch
    | s == (BEIdent "") && r /= send= do 
                        let f _ = Just (r, send)
                        modifyChanState ch { chsChans = M.alter f chan (chsChans ch) }
    | send == r = return $ ()
    | otherwise = lift $ Bad $ "Already added sender" ++ (show chan) ++ (show send)


addRecvToChanState :: BEExp -> GenErr ()
addRecvToChanState chan =  do
    recv <- getCurrentProc
    ch <- getChanState
    let a = M.lookup chan (chsChans ch)
    case a of
        Just d -> modifyRecvInChanState chan recv d ch
        Nothing -> lift $ Bad $ "Channel not decleared" ++ show chan

modifyRecvInChanState :: BEExp -> BEIdent -> (BEIdent, BEIdent) -> ChanState -> GenErr ()
modifyRecvInChanState chan recv (r, s) ch
    | r == (BEIdent "") && s /= recv  = do 
                        let f _ = Just (recv, s)
                        modifyChanState ch { chsChans = M.alter f chan (chsChans ch) }
    | recv == s = return $ ()
    | otherwise = lift $ Bad $ "Already added receiver" ++ (show chan) ++ (show recv)


chanMapContainsProc :: BEIdent -> GenErr Bool
chanMapContainsProc proc = do 
    ch <- getChanState
    let (r,s) = unzip $ M.elems (chsChans ch)
    let rContains = L.any (\e -> e == proc) r
    let sContains = L.any (\e -> e == proc) s
    return $ rContains || sContains

createChanDep :: GenErr ()
createChanDep = getChanState >>= \st -> getChanUsageLists st M.empty

getChanUsageLists :: ChanState -> (M.Map BEIdent [BEExp]) -> GenErr ()
getChanUsageLists ch depMap = if (chsChans ch) /= M.empty
    then do
        let (chan, (recvProc, sendProc)) = M.elemAt 0 (chsChans ch)
        recvMap <- addToDepMap recvProc chan depMap
        sendMap <- addToDepMap sendProc chan depMap
        getChanUsageLists ChanState { chsChans = M.deleteAt 0 (chsChans ch) } $ M.unionWith L.union recvMap sendMap
     else do
        let list = M.toList depMap
        _ <- mapM createChanDepFunc list
        return $ ()

addToDepMap :: BEIdent -> BEExp -> M.Map BEIdent [BEExp] -> GenErr (M.Map BEIdent [BEExp])
addToDepMap proc chan depMap = if M.member proc depMap
                            then case M.lookup proc depMap of
                                    Just _ -> 
                                        let f a = Just (chan:a) in
                                        return $ M.update f proc depMap
                                    Nothing -> lift $ Bad $ "Corrupt chan dep list " ++ (show depMap)
                            else return $ M.insert proc [chan] depMap

createChanDepFunc :: (BEIdent, [BEExp]) -> GenErr ()
createChanDepFunc (BEIdent "", _) = return $ ()
createChanDepFunc (procName, list) = do
    depStatements <- mapM createDepFunc list
    let comp = BECompound $ depStatements
    let funcName = genChanRegProcFunc procName
    let proc = (helpCreateStructPointerType genProcStruct, genProc)
    addToSource $ helpCreateFunc helpCreateVoidType funcName [proc] comp
    addToHeader $ helpCreateFuncProto helpCreateVoidType (genChanRegProcFunc procName) [proc] 

createDepFunc :: BEExp -> GenErr BEStatement
createDepFunc chan = return $ helpCallFuncExp genChanAddDepToChan 
                                    [chan, helpCreateVar genProc]
