module GenErr where 

import Control.Monad.State
import Data.Map as M
import Backend.Abs
import Parser.ErrM

data GenState = GenState {
  gsHeaderGen :: [BEGlobal],
  gsSourceGen :: [BEGlobal],
  gsVarCount :: Int,
  gsPcCount :: Int,
  gsProcCount :: Int,
  gsDeadline :: Int,
  gsCurrentProc :: BEIdent,
  gsCurrentLoop :: (BEExp, Int, Int)
} deriving (Eq, Ord, Show)

data VarState = VarState {
    vsLocal :: [(BEExp, Int)]
} deriving (Eq, Ord, Show)

data ChanState = ChanState {
    chsChans :: M.Map BEExp (BEIdent, BEIdent)
} deriving (Eq, Ord, Show)

data CompState = CompState {
  csGenState :: GenState,
  csVarState :: [VarState],
  csChanState :: ChanState
} deriving (Eq, Ord, Show)


type GenErr a = StateT CompState Err a

getGenState :: GenErr GenState
getGenState = get >>= \st -> return $ csGenState st

modifyGenState :: GenState -> GenErr ()
modifyGenState gsState = modify $ \st -> st { csGenState = gsState }

getVarState :: GenErr [VarState]
getVarState = get >>= \st -> return $ csVarState st

modifyVarState :: [VarState] -> GenErr ()
modifyVarState vsState = modify $ \st -> st { csVarState = vsState }

getChanState :: GenErr ChanState
getChanState = get >>= \st -> return $ csChanState st

modifyChanState :: ChanState -> GenErr ()
modifyChanState chsState = modify $ \st -> st { csChanState = chsState }
