module GeneratorState where

import Control.Monad.State
import Data.Char

import Backend.Abs
import Parser.ErrM

import GenErr 

addToHeader :: BEGlobal -> GenErr ()
addToHeader str = getGenState >>= \gs -> modifyGenState gs {gsHeaderGen = str : gsHeaderGen gs }

addToSource :: BEGlobal -> GenErr ()
addToSource str =  getGenState >>= \gs -> modifyGenState gs 
                    { gsSourceGen =  gsSourceGen gs ++ [str] }

addToHeadOfSource :: BEGlobal -> GenErr ()
addToHeadOfSource str = getGenState >>= \gs -> modifyGenState gs { gsSourceGen = [str] ++ gsSourceGen gs }

getProgram :: GenErr BEProgram 
getProgram  = getGenState >>= \gs -> return $ BEProgram $ (gsHeaderGen gs) ++ (gsSourceGen gs)

getCounters :: GenErr (Int, Int)
getCounters = getGenState >>= \gs -> return $ (gsVarCount gs, gsPcCount gs)

setCounters :: (Int, Int) -> GenErr ()
setCounters (varCount, pc) = do 
    getGenState >>= \gs -> modifyGenState gs { gsVarCount = varCount }
    getGenState >>= \gs -> modifyGenState gs { gsPcCount = pc }

resetCounters :: GenErr ()
resetCounters = resetVarCount >> resetPcCount

getVarCount :: GenErr Int
getVarCount =  getGenState >>= \gs -> return $ gsVarCount gs

incVarCount :: GenErr Int
incVarCount = do
  getGenState >>= \gs -> modifyGenState gs { gsVarCount = gsVarCount gs + 1 } 
  getVarCount

incVarCountWith :: Int -> GenErr ()
incVarCountWith size = getGenState >>=  \gs -> modifyGenState gs 
                        { gsVarCount = gsVarCount gs + size }

resetVarCount :: GenErr ()
resetVarCount = getGenState >>= \gs -> modifyGenState gs { gsVarCount = 0 }

getPcCount :: GenErr Int
getPcCount = getGenState >>= \gs -> return $ gsPcCount gs

incPcCount :: GenErr Int
incPcCount = do 
  getGenState >>= \gs -> modifyGenState gs { gsPcCount = gsPcCount gs + 1 }
  getPcCount

resetPcCount :: GenErr ()
resetPcCount = getGenState >>= \gs -> modifyGenState gs { gsPcCount = 0 }

genProcessName :: GenErr String
genProcessName = do
    num <- getGenState >>= \gs -> return $ gsProcCount gs
    getGenState >>= \gs -> modifyGenState gs { gsProcCount = gsProcCount gs + 1 }
    return $ "Par_proc" ++ [intToDigit num]

setDeadline :: Integer -> GenErr ()
setDeadline deadline = getGenState >>= \gs -> modifyGenState gs { gsDeadline = fromInteger deadline }

getDeadline :: GenErr Integer
getDeadline = getGenState >>= \gs -> return $ toInteger $ gsDeadline gs

setCurrentProc :: BEIdent -> GenErr ()
setCurrentProc ident = getGenState >>= \gs -> modifyGenState gs { gsCurrentProc = ident }

getCurrentProc :: GenErr BEIdent
getCurrentProc = getGenState >>= \gs -> return $ gsCurrentProc gs

setCurrentLoop :: BEExp -> Int -> Int -> GenErr ()
setCurrentLoop var start to = case var of
    BEEVar _ -> getGenState >>= \gs -> modifyGenState gs { gsCurrentLoop = (var, start, to) }
    _ -> lift $ Bad $ "Loop counter must be a variable not " ++ (show var)

resetCurrentLoop :: GenErr()
resetCurrentLoop = setCurrentLoop (BEEVar (BEIdent "")) 0 0

getCurrentLoop :: GenErr (BEExp, Int, Int)
getCurrentLoop = getGenState >>= \gs -> return $ gsCurrentLoop gs
