module Help where

import Backend.Abs

import Includes
import GenErr

helpCreateVar :: String -> BEExp
helpCreateVar str = BEEVar (BEIdent str)

helpDeclVar :: BEPrimitive -> String -> BEStatement
helpDeclVar prim str= BEDeclaration $ BESemicolonDecl $ BEDeclNoInit 
                (BEType [] prim [])
                (BEDeclIdentVar (BEIdent str))
                [] 

helpDeclAssignVarVar:: String -> String -> BEPrimitive -> [BEStatement]
helpDeclAssignVarVar l r prim = [decl, assign] where 
  decl = helpDeclVar prim l
  assign = helpAssignVarVar l r

helpDeclAssignVarFunc :: String -> String -> [BEExp] -> BEPrimitive -> [BEStatement]
helpDeclAssignVarFunc l func args prim = [decl, assign]  where 
    decl = helpDeclVar prim l
    assign = helpAssignVarFunc (helpCreateVar l) func args

helpDeclAssignPointVarFunc :: String -> String -> [String] -> BEPrimitive -> [BEStatement]
helpDeclAssignPointVarFunc l func args prim = [decl, assign] where 
    decl = BEDeclaration $ BESemicolonDecl $ BEDeclNoInit 
                    (BEType [] prim [BEPointerSpec])
                    (BEDeclIdentVar (BEIdent l))
                    []
    assign =  helpAssignVarFunc (helpCreateVar l) func $ map helpCreateVar args

helpDeclAssignVarPoint :: BEPrimitive -> String -> String -> String -> [BEStatement]
helpDeclAssignVarPoint prim l p r = [decl, assign] where 
    decl = helpDeclVar prim l
    assign = helpAssignPointVar p l r

helpAssignVarFunc :: BEExp -> String -> [BEExp] -> BEStatement 
helpAssignVarFunc l func args = BEStatementExpression $ BEEAssign l BEAssign
                    $ BEEFunk (BEEVar (BEIdent func)) $ args

helpAssignVarExpr :: String -> BEExp -> BEStatement
helpAssignVarExpr l r = BEStatementExpression $ BEEAssign 
                    (BEEVar (BEIdent l)) BEAssign $ r

helpAssignVarVar :: String -> String -> BEStatement
helpAssignVarVar l r = helpAssignVarExpr l $ BEEVar $ BEIdent r

helpAssignPointVar :: String -> String -> String -> BEStatement
helpAssignPointVar p l r = BEStatementExpression $ BEEAssign 
                                (BEEVar ( BEIdent r ))
                                BEAssign $ BEEPoint (BEEVar (BEIdent p)) $ BEEVar $ BEIdent l
helpCreateVoidType :: BEType
helpCreateVoidType = BEType [] BEPrimitiveVoid []

helpCreateIntType :: BEType
helpCreateIntType =  BEType [] (BEPrimitiveInt) [] 

helpCreateEnumType :: String -> BEType
helpCreateEnumType str =  BEType [] (BEPrimitiveEnum (BEIdent str)) [] 

helpCreateStructPointerType :: String -> BEType
helpCreateStructPointerType str = BEType [] (BEPrimitiveStruct (BEIdent str)) [BEPointerSpec] 

helpCreateDeclList :: (BEType, String) -> BECommaDecl
helpCreateDeclList (decl,ident) = BECommaDecl $ 
                                    BEDeclNoInit decl (BEDeclIdentVar (BEIdent ident)) []

helpCreateCase :: Integer -> BEStatement
helpCreateCase n = BECaseLabel $ BEEConst $ BEConstLiteralInt n

helpCreateFor :: BEExp -> Integer -> Integer -> BECompound -> BEStatement
helpCreateFor var ini to block = let
  assign = BEEAssign var BEAssign $ BEEConst $ BEConstLiteralInt ini
  cond = BEELT var $ BEEConst $ BEConstLiteralInt to
  inc = BEEPostInc $ var in 
  BEFor assign cond inc block

helpCreateFuncStr :: BEType -> String -> [(BEType,String)]-> BECompound -> BEGlobal
helpCreateFuncStr rtype ident args block = helpCreateFunc rtype (BEIdent ident) args block

helpCreateFunc :: BEType -> BEIdent -> [(BEType, String)] -> BECompound -> BEGlobal
helpCreateFunc rtype ident args block = BEGlobalFunc $ BEFunction rtype ident
        (map helpCreateDeclList args) block

helpCreateFuncProtoStr :: BEType -> String -> [(BEType,String)] -> BEGlobal
helpCreateFuncProtoStr rtype ident args = helpCreateFuncProto rtype (BEIdent ident) args

helpCreateFuncProto :: BEType -> BEIdent -> [(BEType, String)] -> BEGlobal
helpCreateFuncProto rtype ident args = BEGlobalFunc $ BEFunctionProto rtype ident 
        (map helpCreateDeclList args)

helpCreateGoto :: String -> BEStatement
helpCreateGoto str = BEGoto $ BEIdent str

helpCallFuncExp :: String -> [BEExp] -> BEStatement
helpCallFuncExp func args = BEStatementExpression $ BEEFunk (BEEVar (BEIdent func)) $ args

helpCallFuncStr :: String -> [String] -> BEStatement
helpCallFuncStr func args = BEStatementExpression $ BEEFunk (BEEVar (BEIdent func))
                            $ map helpCreateVar args

helpCreateNewProcChanAndProcDep :: String -> (BEIdent -> GenErr Bool) -> GenErr BEStatement
helpCreateNewProcChanAndProcDep  procName f = do
    check <- f $ BEIdent procName 
    if check
        then helpCreateNewProcPure procName [genStack, genChanRegProcFuncStr procName, genProc]
        else helpCreateNewProcPure procName [genStack, genNull, genProc];

helpCreateNewProcPure :: String -> [String] -> GenErr BEStatement
helpCreateNewProcPure procName args =  
    return $ helpCallFuncStr genNewProc $ 
        [procName, (genProcessVarName procName),(genDeadline procName)] ++ args;

helpGetAddress :: BEExp -> BEExp
helpGetAddress var = BEEPreOp BEAddress $ var

helpLock :: BEExp -> BEExp -> BEStatement 
helpLock struct lock = helpCallFuncExp genMutexLock [BEEPoint struct lock]

helpUnlock :: BEExp -> BEExp -> BEStatement
helpUnlock struct lock = helpCallFuncExp genMutexUnlock [BEEPoint struct lock]
