module Options where 

import System 
import System.Console.GetOpt

data Options = Options {
  optInput :: IO String,
  optOutput :: String -> IO ()
  }

defaultOptions :: Options
defaultOptions = Options {
  optInput = getContents,
  optOutput = putStr 
}

options :: [OptDescr (Options -> IO Options)]
options = [
    Option ['V'] ["version"] (NoArg showVersion)         "show version number",
    Option ['i'] ["input"]   (ReqArg readInput "FILE")   "input file to read",
    Option ['o'] ["output"]  (ReqArg writeOutput "FILE") "output file to write"
  ]

showVersion :: Options -> IO Options
showVersion _ = do 
  putStrLn "Jon Tore Hafstad comp v0.1"
  exitWith ExitSuccess

readInput :: String -> Options -> IO Options
readInput arg opt = return opt {optInput = readFile arg}

writeOutput :: String -> Options -> IO Options
writeOutput arg opt = return opt {optOutput = writeFile arg}

parseOptions :: IO Options
parseOptions = do
 args <- getArgs
 let (actions, _, _) =  getOpt Permute options args 
 opts <- foldl (>>=) (return defaultOptions) actions 
 return opts
