module Parser.Print where

-- pretty-printer generated by the BNF converter

import Parser.Abs
import Char

-- the top-level printing method
printTree :: Print a => a -> String
printTree = render . prt 0

type Doc = [ShowS] -> [ShowS]

doc :: ShowS -> Doc
doc = (:)

render :: Doc -> String
render d = rend 0 (map ($ "") $ d []) "" where
  rend i ss = case ss of
    "["      :ts -> showChar '[' . rend i ts
    "("      :ts -> showChar '(' . rend i ts
    "{"      :ts -> showChar '{' . new (i+1) . rend (i+1) ts
    "}" : ";":ts -> new (i-1) . space "}" . showChar ';' . new (i-1) . rend (i-1) ts
    "}"      :ts -> new (i-1) . showChar '}' . new (i-1) . rend (i-1) ts
    ";"      :ts -> showChar ';' . new i . rend i ts
    t  : "," :ts -> showString t . space "," . rend i ts
    t  : ")" :ts -> showString t . showChar ')' . rend i ts
    t  : "]" :ts -> showString t . showChar ']' . rend i ts
    t        :ts -> space t . rend i ts
    _            -> id
  new i   = showChar '\n' . replicateS (2*i) (showChar ' ') . dropWhile isSpace
  space t = showString t . (\s -> if null s then "" else (' ':s))

parenth :: Doc -> Doc
parenth ss = doc (showChar '(') . ss . doc (showChar ')')

concatS :: [ShowS] -> ShowS
concatS = foldr (.) id

concatD :: [Doc] -> Doc
concatD = foldr (.) id

replicateS :: Int -> ShowS -> ShowS
replicateS n f = concatS (replicate n f)

-- the printer class does the job
class Print a where
  prt :: Int -> a -> Doc
  prtList :: [a] -> Doc
  prtList = concatD . map (prt 0)

instance Print a => Print [a] where
  prt _ = prtList

instance Print Char where
  prt _ s = doc (showChar '\'' . mkEsc '\'' s . showChar '\'')
  prtList s = doc (showChar '"' . concatS (map (mkEsc '"') s) . showChar '"')

mkEsc :: Char -> Char -> ShowS
mkEsc q s = case s of
  _ | s == q -> showChar '\\' . showChar s
  '\\'-> showString "\\\\"
  '\n' -> showString "\\n"
  '\t' -> showString "\\t"
  _ -> showChar s

prPrec :: Int -> Int -> Doc -> Doc
prPrec i j = if j<i then parenth else id


instance Print Integer where
  prt _ x = doc (shows x)


instance Print Double where
  prt _ x = doc (shows x)



instance Print CapIdent where
  prt _ (CapIdent i) = doc (showString i)


instance Print LowIdent where
  prt _ (LowIdent i) = doc (showString i)



instance Print ProgramSpec where
  prt i e = case e of
   Program globdecls -> prPrec i 0 (concatD [prt 0 globdecls])


instance Print GlobDecl where
  prt i e = case e of
   GlobalProcessDecl capident deadline singleblock -> prPrec i 0 (concatD [doc (showString "PROC") , prt 0 capident , doc (showString ":") , prt 0 deadline , doc (showString ":") , prt 0 singleblock])
   GlobalChannelDecl chan -> prPrec i 0 (concatD [prt 0 chan , doc (showString ";")])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , prt 0 xs])

instance Print SingleBlock where
  prt i e = case e of
   SingleBlock vars stmtcompound -> prPrec i 0 (concatD [doc (showString "{") , prt 0 vars , prt 0 stmtcompound , doc (showString "}")])


instance Print StmtCompound where
  prt i e = case e of
   StmtFor order lowident assignop n0 n stmtblock -> prPrec i 0 (concatD [prt 0 order , doc (showString "FOR") , prt 0 lowident , prt 0 assignop , prt 0 n0 , doc (showString "TO") , prt 0 n , doc (showString ":") , prt 0 stmtblock])
   StmtOrder order stmtblock -> prPrec i 0 (concatD [prt 0 order , prt 0 stmtblock])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , prt 0 xs])

instance Print StmtBlock where
  prt i e = case e of
   StmtBlock stmts -> prPrec i 0 (concatD [doc (showString "{") , prt 0 stmts , doc (showString "}")])


instance Print Order where
  prt i e = case e of
   OrderPar  -> prPrec i 0 (concatD [doc (showString "PAR")])
   OrderSeq  -> prPrec i 0 (concatD [doc (showString "SEQ")])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , prt 0 xs])

instance Print DeadLine where
  prt i e = case e of
   DeadLineS n -> prPrec i 0 (concatD [prt 0 n , doc (showString "S")])
   DeadLineMs n -> prPrec i 0 (concatD [prt 0 n , doc (showString "Ms")])
   DeadLineUs n -> prPrec i 0 (concatD [prt 0 n , doc (showString "Us")])
   DeadLineNs n -> prPrec i 0 (concatD [prt 0 n , doc (showString "Ns")])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , prt 0 xs])

instance Print Stmt where
  prt i e = case e of
   StmtAssign lowident assignop expr -> prPrec i 0 (concatD [prt 0 lowident , prt 0 assignop , prt 0 expr , doc (showString ";")])
   StmtArrayAssign lowident array assignop expr -> prPrec i 0 (concatD [prt 0 lowident , prt 0 array , prt 0 assignop , prt 0 expr , doc (showString ";")])
   StmtDeSchedule  -> prPrec i 0 (concatD [doc (showString "DeSchedule") , doc (showString ";")])
   StmtCall capident -> prPrec i 0 (concatD [prt 0 capident , doc (showString ";")])
   StmtExpr expr -> prPrec i 0 (concatD [prt 0 expr , doc (showString ";")])
   StmtWork  -> prPrec i 0 (concatD [doc (showString "Work") , doc (showString ";")])
   StmtCompound stmtcompound -> prPrec i 0 (concatD [prt 0 stmtcompound])
   StmtPrint str exprs -> prPrec i 0 (concatD [doc (showString "Print") , prt 0 str , prt 0 exprs])
   StmtChanCom chancom -> prPrec i 0 (concatD [prt 0 chancom , doc (showString ";")])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , prt 0 xs])

instance Print Chan where
  prt i e = case e of
   ChanVar lowident -> prPrec i 0 (concatD [doc (showString "CHAN") , prt 0 lowident])
   ChanArray lowident array -> prPrec i 0 (concatD [doc (showString "CHAN") , prt 0 lowident , prt 0 array])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print Var where
  prt i e = case e of
   Var lowident -> prPrec i 0 (concatD [doc (showString "VAR") , prt 0 lowident])
   VarArray lowident array -> prPrec i 0 (concatD [doc (showString "VAR") , prt 0 lowident , prt 0 array])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print Array where
  prt i e = case e of
   Array expr -> prPrec i 0 (concatD [doc (showString "[") , prt 0 expr , doc (showString "]")])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , prt 0 xs])

instance Print AssignOp where
  prt i e = case e of
   Assign  -> prPrec i 0 (concatD [doc (showString "=")])


instance Print ChanCom where
  prt i e = case e of
   ChanComSend chanexpr expr -> prPrec i 0 (concatD [prt 0 chanexpr , doc (showString "!") , prt 0 expr])
   ChanComRecv chanexpr expr -> prPrec i 0 (concatD [prt 0 chanexpr , doc (showString "?") , prt 0 expr])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print ChanExpr where
  prt i e = case e of
   ChanExpr lowident -> prPrec i 0 (concatD [prt 0 lowident])
   ChanExprArray lowident expr -> prPrec i 0 (concatD [prt 0 lowident , doc (showString "[") , prt 0 expr , doc (showString "]")])

  prtList es = case es of
   [] -> (concatD [])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])

instance Print Expr where
  prt i e = case e of
   Plus expr0 expr -> prPrec i 1 (concatD [prt 1 expr0 , doc (showString "+") , prt 2 expr])
   Min expr0 expr -> prPrec i 1 (concatD [prt 1 expr0 , doc (showString "-") , prt 2 expr])
   ExprArray expr0 expr -> prPrec i 1 (concatD [prt 2 expr0 , doc (showString "[") , prt 2 expr , doc (showString "]")])
   Numb n -> prPrec i 4 (concatD [prt 0 n])
   ExprVar lowident -> prPrec i 5 (concatD [prt 0 lowident])

  prtList es = case es of
   [] -> (concatD [])
   [] -> (concatD [])
   [x] -> (concatD [prt 0 x])
   x:xs -> (concatD [prt 0 x , doc (showString ";") , prt 0 xs])
   x:xs -> (concatD [prt 0 x , doc (showString ",") , prt 0 xs])


