#include <stdlib.h>
#include <stdio.h>

#include "linkedList.h"
#include "type.h"

struct LinkedListProc* linkedListProcInit(){
    struct LinkedListProc *list = malloc(sizeof(struct LinkedListProc));
    list->start = NULL;
    list->end = NULL;
    list->lock = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    pthread_mutex_init(list->lock, NULL);
    return list;
}

void linkedListProcFree(struct LinkedListProc *list) {
   if(!list) {
       return;
   }
   pthread_mutex_lock(list->lock);
   list->start = NULL;
   list->end = NULL;
   pthread_mutex_unlock(list->lock);
   pthread_mutex_destroy(list->lock);
   list->lock = NULL;
   free(list->lock);
   free(list);
}

void linkedListProcPrint(struct LinkedListProc *list) {
    if(!list)
        return;
    struct LinkedListProcNode *tmp = list->start;
    while(tmp) {
        if(tmp->proc){
            printf("List %d pid %llu\n", (int)list, tmp->proc->pid);
        }
        tmp = tmp->next;
    }
}

void linkedListProcQueue(struct LinkedListProc *list, struct Proc *proc) {
    
    if(!proc || !list) {
        return;
    }
    
    pthread_mutex_lock(list->lock);
    struct LinkedListProcNode *tmp = malloc(sizeof(struct LinkedListProcNode));
    tmp->next = NULL;
    tmp->proc = proc;

    if(!(list->start) || !(list->end)) {
        list->start = tmp;
        list->end = tmp;
    }
    else {
        list->end->next = tmp;
        list->end = tmp;
    }
    
    pthread_mutex_unlock(list->lock);
}

struct Proc* linkedListProcDequeue(struct LinkedListProc *list) {
    if(!list) {
        return NULL;
    }

    pthread_mutex_lock(list->lock);
    
    if(!list->start) {
        pthread_mutex_unlock(list->lock);
        return NULL;
    }
    

    struct Proc *proc = list->start->proc;
    struct LinkedListProcNode *tmp = list->start;

    if((list->start == list->end) || tmp == list->end || !(tmp->next)) {
        list->start = NULL;
        list->end = NULL;
    }
    else if (tmp) {
        list->start = tmp->next;
    }
    
    free(tmp);
      
    pthread_mutex_unlock(list->lock);

    return proc;
}

struct LinkedListChan *linkedListChanInit(){
    struct LinkedListChan *list = malloc(sizeof(struct LinkedListChan));
    list->start = NULL;
    list->end = NULL;
    list->lock = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    pthread_mutex_init(list->lock, NULL);
    
    return list;
}

void linkedListChanFree(struct LinkedListChan *list) {
   if(!list) {
      return; 
   }

   pthread_mutex_lock(list->lock);
   list->start = NULL;
   list->end = NULL;
   pthread_mutex_unlock(list->lock);
   pthread_mutex_destroy(list->lock);
   list->lock = NULL;
   free(list->lock);
   free(list);
}

void linkedListChanQueue(struct LinkedListChan *list, struct Channel *chan) {
    
    if(!chan || !list) {
        return;
    }

    pthread_mutex_lock(list->lock);

    struct LinkedListChanNode *tmp = malloc(sizeof(struct LinkedListChanNode));
    tmp->next = NULL;
    tmp->chan = chan;

    if(!(list->start) || !(list->end)) {
        list->start = tmp;
        list->end = tmp;
    }
    else {
        list->end->next = tmp;
        list->end = tmp;
    }
    pthread_mutex_unlock(list->lock);
}

struct Channel* linkedListChanDequeue(struct LinkedListChan *list) {
    if(!list) {
        return NULL;
    }
    
    pthread_mutex_lock(list->lock);
    
    if(!list->start) {
        pthread_mutex_unlock(list->lock);
        return NULL;
    }

    struct Channel *chan = list->start->chan;
    struct LinkedListChanNode *tmp = list->start;
   
    if((list->start == list->end) || (tmp == list->end || !(tmp->next))) {
        list->start = NULL;
        list->end = NULL;
    }
    else if (tmp) {
       list->start = tmp->next;
    }
    free(tmp);
      
    pthread_mutex_unlock(list->lock);

    return chan;
}


struct LinkedListStack *linkedListStackInit(){
    struct LinkedListStack *list = malloc(sizeof(struct LinkedListStack));
    list->start = NULL;
    list->end = NULL;
    list->lock = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    pthread_mutex_init(list->lock, NULL);
    
    return list;
}

void linkedListStackFree(struct LinkedListStack *list) {
   if(!list) {
      return; 
   }

   pthread_mutex_lock(list->lock);
   list->start = NULL;
   list->end = NULL;
   pthread_mutex_unlock(list->lock);
   pthread_mutex_destroy(list->lock);
   free(list);
}

void linkedListStackQueue(struct LinkedListStack *list, struct Stack *stack) {
    if(!list) {
        return;
    }

    pthread_mutex_lock(list->lock);
    
    if(!stack ) {
        pthread_mutex_unlock(list->lock);
        return;
    }

    struct LinkedListStackNode *tmp = malloc(sizeof(struct LinkedListStackNode));
    tmp->next = NULL;
    tmp->stack = stack;

    if(!(list->start) || !(list->end)) {
        list->start = tmp;
        list->end = tmp;
    }
    else {
        list->end->next = tmp;
        list->end = tmp;
    }
    pthread_mutex_unlock(list->lock);
}

struct Stack* linkedListStackDequeue(struct LinkedListStack *list) {
    if(!list) {
        return NULL;
    }
    
    pthread_mutex_lock(list->lock);
    
    if(!list->start) {
        pthread_mutex_unlock(list->lock);
        return NULL;
    }

    struct Stack *stack = list->start->stack;
    struct LinkedListStackNode *tmp = list->start;
   
    if((list->start == list->end) || (tmp == list->end || !(tmp->next))) {
        list->start = NULL;
        list->end = NULL;
    }
    else if (tmp) {
       list->start = tmp->next;
    }
    free(tmp);
      
    pthread_mutex_unlock(list->lock);

    return stack;
}
