#include <stdlib.h>
#include <pthread.h>
#include <time.h>

#include <rbtree/rbtree.h>
#include <rbQueue.h>
#include <procQueue.h>

struct ProcQueue *makeProcQueue(void) {
    struct ProcQueue *queue = malloc(sizeof(struct ProcQueue));
    queue->root = (struct rb_root *)malloc(sizeof(struct rb_root));
    queue->queueLock = (pthread_mutex_t *)malloc(sizeof(pthread_mutex_t));
    pthread_mutex_init(queue->queueLock, NULL);
    return queue;
}

void freeProcQueue(struct ProcQueue *queue) {
    pthread_mutex_destroy(queue->queueLock);
    rbFinish(queue->root);
    free(queue->root);
    free(queue->queueLock);
    free(queue);
}

void queueProc(struct Proc *proc, struct ProcQueue *procQueue, 
        void (*fp)(struct rb_root *, struct rbTask *)) 
{
	struct rbTask *newElem = malloc(sizeof(struct rbTask));
    newElem->proc = proc;
    newElem->pid = proc->pid;
    
    pthread_mutex_lock(procQueue->queueLock);
    fp(procQueue->root, newElem);
    pthread_mutex_unlock(procQueue->queueLock);

}

void queueReadyProc(struct Proc *proc, struct ProcQueue *procQueue) {
    queueProc(proc, procQueue, &rbInsertDeadline);
}

void queueWaitingProc(struct Proc *proc, struct ProcQueue *procQueue) {
    queueProc(proc, procQueue, &rbInsertPid); 
}

struct Proc *dequeueProc(struct ProcQueue *procQueue,
                     unsigned long long n,
                     struct rbTask * (*fp)(struct rb_root *, unsigned long long n)) {

    pthread_mutex_lock(procQueue->queueLock);
	struct Proc *proc = NULL;
    
    struct rbTask *elem = fp(procQueue->root, n);
    if(elem) {
	    proc = elem->proc;
        rbDeleteFull(procQueue->root, elem);
    }
    pthread_mutex_unlock(procQueue->queueLock);
	return proc;
}

struct Proc *dequeueReadyProc(struct ProcQueue *procQueue) {
    return dequeueProc(procQueue, 0, &rbGetLowestDeadline);
}

struct Proc *dequeueWaitingProc(struct ProcQueue *procQueue, unsigned long long pid) {
    return dequeueProc(procQueue, pid, &rbSearchPid);
}

void removeReadyProcFromQueue(struct ProcQueue *procQueue,
                        unsigned long long deadline,
                        unsigned long long pid) {
    struct Proc *tmp = rbDeleteDeadline(procQueue->root, deadline, pid);
    if(!tmp){
        free(tmp);
    }
}

void removeWaitingProcFromQueue(struct ProcQueue *procQueue,
                        unsigned long long pid) {
    struct Proc *tmp = rbDeletePid(procQueue->root, pid);
    if(!tmp)
        free(tmp);
}
